==> 0.0.1.tar.gz: lib/src/fi18n_impl.dart <==
import 'dart:collection';
import 'dart:convert';

import 'package:fi18n/src/models/file_meta.dart';

Pattern twoCharsLower = RegExp('^[a-z]{2}\$');
Pattern twoCharsUpper = RegExp('^[A-Z]{2}\$');

String generateSClass({String defaultArbContent,
  List<String> supportedLocales,
  String generatedClassName}) {
  final messages = json.decode(defaultArbContent) as LinkedHashMap<String, dynamic>;

  final script = StringBuffer();

  void writeLine(String data) => script.writeln(data);

  writeLine('class $generatedClassName implements WidgetsLocalizations {');
  writeLine('  const $generatedClassName();');
  writeLine('');
  writeLine('  static $generatedClassName current;');
  writeLine('');
  writeLine('  static const AppLocalizationDelegate delegate = AppLocalizationDelegate();');
  writeLine('');
  writeLine('  static $generatedClassName of(BuildContext context) => Localizations.of<$generatedClassName>(context, $generatedClassName);');
  writeLine('');
  writeLine('  @override');
  writeLine('  TextDirection get textDirection => TextDirection.ltr;');
  writeLine('');
  messages.forEach((k, v) {
    writeLine("  String get $k => '$v';");
  });
  writeLine('}');

  return script.toString();
}

String generateLanguageScript(LinkedHashMap<String, dynamic> messages,
    {String locale, String generatedClassName}) {
  final script = StringBuffer();

  void writeLine(String data) => script.writeln(data);

  String objectClassName = '\$$locale';
  String textDirection = _rtl.contains(locale.split('_')) ? 'rlt' : 'ltr';

  writeLine('class $objectClassName extends $generatedClassName {');
  writeLine('  const $objectClassName();');
  writeLine('');
  writeLine('  @override');
  writeLine('  TextDirection get textDirection => TextDirection.$textDirection;');
  writeLine('');
  messages.forEach((k, v) {
    writeLine('  @override');
    writeLine("  String get $k => '$v';");
  });
  writeLine('');
  writeLine('}');
  return script.toString();
}

FileMeta generateArbFileMetaFromArb({String filename, String defaultLocaleName}) {
  final name = filename.replaceAll('.arb', '');

  final nameParts = name.split('_');
  if (nameParts.isEmpty) {
    throw Exception('Wrong file name: $name');
  }

  final result = FileMeta();

  result.objectName =  _firstCharUpper(nameParts[0]);

  if (nameParts.last == defaultLocaleName) {
    result.isDefault = true;
    result.localeName = defaultLocaleName;
    //TODO(pedromassango): extract language code from locale name
    result.languageCode = defaultLocaleName;
    return result;
  } else {
    result.fileName = '${nameParts[0]}.dart';
    result.isDefault = false;

    if (nameParts.length > 3) {
      throw Exception('Wrong file name: $name');
    }

    if (nameParts.length >= 2) {
      final languageCode = nameParts[1];
      if (twoCharsLower.allMatches(languageCode).length != 1) {
        throw Exception('Wrong language code "$languageCode" in file name "$name". Language code must match $twoCharsLower');
      }
      result.languageCode = languageCode;
      result.localeName = languageCode;
    }
    if (nameParts.length == 3) {
      final countryCode = nameParts[2];
      if (twoCharsUpper.allMatches(countryCode).length != 1) {
        throw Exception('Wrong country code "$countryCode" in file name'
            ' "$filename". Country code must match $twoCharsUpper');
      }
      result.localeName = '${result.languageCode}_$countryCode';
    }
    result.objectName = '${result.objectName}_${result.localeName}';
    return result;
  }
}

String generateDelegateScript({String defaultArbContent,
  List<String> supportedLocales, String objectClassName}) {

  final script = StringBuffer();

  void writeLine(String data) => script.writeln(data);

  writeLine('class AppLocalizationDelegate extends LocalizationsDelegate<$objectClassName> {');
  writeLine('  const AppLocalizationDelegate();');
  writeLine('');
  writeLine('  List<Locale> get supportedLocales {');
  writeLine('    return const <Locale>[');
  supportedLocales.forEach((localeName) {
    writeLine("      Locale.fromSubtags(languageCode: '$localeName'),");
  });
  writeLine('    ];');
  writeLine('  }');
  writeLine('');
  writeLine(r'  LocaleListResolutionCallback listResolution({Locale fallback}) {');

  script
    ..writeln(r'    return (List<Locale> locales, Iterable<Locale> supported) {')
    ..writeln(r'      if (locales == null || locales.isEmpty) {')
    ..writeln(r'        return fallback ?? supported.first;')
    ..writeln(r'      } else {')
    ..writeln(r'        return _resolve(locales.first, fallback, supported);')
    ..writeln(r'      }')
    ..writeln(r'    };')
    ..writeln(r'  }')
    ..writeln()
    ..writeln(r'  LocaleResolutionCallback resolution({Locale fallback}) {')
    ..writeln(r'    return (Locale locale, Iterable<Locale> supported) {')
    ..writeln(r'      return _resolve(locale, fallback, supported);')
    ..writeln(r'    };')
    ..writeln(r'  }')
    ..writeln()
    ..writeln(r'  @override')
    ..writeln('  Future<$objectClassName> load(Locale locale) {')
    ..writeln(r'    final String lang = getLang(locale);')
    ..writeln(r'    if (lang != null) {')
    ..writeln(r'      switch (lang) {');

  for (int i = 0; i < supportedLocales.length; i++) {
    final String language = supportedLocales[i];

    // for hebrew iw==he
    if (language.startsWith('iw')) {
      script
        ..writeln(r'        case "iw_IL":')
        ..writeln(r'        case "he_IL":')
        ..writeln('          $objectClassName.current = const \$he_IL();')
        ..writeln('          return SynchronousFuture<$objectClassName>($objectClassName.current);');
    } else {
      script
        ..writeln('        case "$language":')
        ..writeln('          $objectClassName.current = const \$$language();')
        ..writeln('          return SynchronousFuture<$objectClassName>($objectClassName.current);');
    }
  }

  script
    ..writeln(r'        default:')
    ..writeln(r'          //...')
    ..writeln(r'      }')
    ..writeln(r'    }')
    ..writeln('    return SynchronousFuture<$objectClassName>(const $objectClassName());')
    ..writeln(r'  }')
    ..writeln();

  writeLine('  @override');
  writeLine('  bool isSupported(Locale locale) => _isSupported(locale);');
  writeLine('');
  writeLine('  @override');
  writeLine('  bool shouldReload(AppLocalizationDelegate old) => false;');
  writeLine('');

  script
  ..writeln(r'  /// Internal method to resolve a locale from a list of locales.')
  ..writeln(r'  Locale _resolve(Locale locale, Locale fallback, Iterable<Locale> supported) {')
  ..writeln(r'    if (locale == null || !_isSupported(locale)) {')
  ..writeln(r'      return fallback ?? supported.first;')
  ..writeln(r'    }')
  ..writeln()
  ..writeln(r'    final Locale languageLocale = Locale(locale.languageCode, "");')
  ..writeln(r'    if (supported.contains(locale)) {')
  ..writeln(r'      return locale;')
  ..writeln(r'    } else if (supported.contains(languageLocale)) {')
  ..writeln(r'      return languageLocale;')
  ..writeln(r'    } else {')
  ..writeln(r'      final Locale fallbackLocale = fallback ?? supported.first;')
  ..writeln(r'      return fallbackLocale;')
  ..writeln(r'    }')
  ..writeln(r'  }');
  writeLine('');
  writeLine('  bool _isSupported(Locale locale) {');
  writeLine('    if (locale != null) {');
  writeLine('      for (Locale supportedLocale in supportedLocales) {');
  writeLine('        if (supportedLocale.languageCode == locale.languageCode) {');
  writeLine('          return true;');
  writeLine('        }');
  writeLine('      }');
  writeLine('    }');
  writeLine('    return false;');
  writeLine('   }');
  writeLine('}');
  writeLine('');
  writeLine('String getLang(Locale locale) => locale == null');
  writeLine('  ? null');
  writeLine('  : locale.countryCode != null && locale.countryCode.isEmpty');
  writeLine('    ? locale.languageCode');
  writeLine('    : locale.toString();');
  writeLine('');

  return script.toString();
}

String _firstCharUpper(String s) {
  return s.replaceRange(0, 1, s[0].toUpperCase());
}

const List<String> _rtl = <String>['ar', 'dv', 'fa', 'ha', 'he', 'iw', 'ji', 'ps', 'ur', 'yi'];

==> 0.0.3.tar.gz: lib/src/fi18n_impl.dart <==
import 'dart:collection';
import 'dart:convert';
import 'package:fi18n/src/models/locale.dart';
import 'package:fi18n/src/models/file_meta.dart';

Pattern twoCharsLower = RegExp('^[a-z]{2}\$');
Pattern twoCharsUpper = RegExp('^[A-Z]{2}\$');

String generateSClass({String defaultArbContent, String generatedClassName}) {
  final messages = json.decode(defaultArbContent) as LinkedHashMap<String, dynamic>;

  final script = StringBuffer();

  void writeLine(String data) => script.writeln(data);

  writeLine('class $generatedClassName implements WidgetsLocalizations {');
  writeLine('  const $generatedClassName();');
  writeLine('');
  writeLine('  static $generatedClassName current;');
  writeLine('');
  writeLine('  static const AppLocalizationDelegate delegate = AppLocalizationDelegate();');
  writeLine('');
  writeLine('  static $generatedClassName of(BuildContext context) => Localizations.of<$generatedClassName>(context, $generatedClassName);');
  writeLine('');
  writeLine('  @override');
  writeLine('  TextDirection get textDirection => TextDirection.ltr;');
  writeLine('');
  messages.forEach((k, v) {
    writeLine("  String get $k => '$v';");
  });
  writeLine('}');

  return script.toString();
}

String generateLanguageScript(
    LinkedHashMap<String, dynamic> messages, {
      Locale locale,
      bool isDefaultLocaleFile,
      String generatedClassName,
    }) {

  final script = StringBuffer();

  void writeLine(String data) => script.writeln(data);

  String objectClassName = '\$${locale.toString()}';
  String textDirection = _rtl.contains(locale.languageCode) ? 'rlt' : 'ltr';

  if(!isDefaultLocaleFile) {
    writeLine('class $objectClassName implements $generatedClassName {');
  } else {
    writeLine('class $objectClassName extends $generatedClassName {');
  }
  writeLine('  const $objectClassName();');
  writeLine('');
  if (!isDefaultLocaleFile) {
    writeLine('  @override');
    writeLine('  TextDirection get textDirection => TextDirection.$textDirection;');
    writeLine('');
    messages.forEach((k, v) {
      writeLine('  @override');
      writeLine("  String get $k => '$v';");
      writeLine('');
    });
  }
  writeLine('}');
  return script.toString();
}

FileMeta generateArbFileMetaFromArb({String filename, String defaultLocaleName}) {
  final name = filename.replaceAll('.arb', '');
  final nameParts = name.split('_');

  if (nameParts.isEmpty || nameParts.length > 3) {
    throw Exception('Wrong file name: $filename');
  }

  final languageCode = nameParts[1];

  if (twoCharsLower.allMatches(languageCode).length != 1) {
    throw Exception('Wrong language code "$languageCode" in file name "$name". Language code must match $twoCharsLower');
  }

  final countryCode = (nameParts.length > 2) ? nameParts[2] : null;

  if (countryCode != null && twoCharsUpper.allMatches(countryCode).length != 1) {
    throw Exception('Wrong country code "$countryCode" in file name'
        ' "$filename". Country code must match $twoCharsUpper');
  }

  final locale = Locale(languageCode, countryCode);

  final localeName = locale.toString();
  final isDefaultLocale = locale.toString() == defaultLocaleName ||
      locale.languageCode == defaultLocaleName;

  final result = FileMeta();
  result.localeName = locale.toString();
  result.objectName = _firstCharUpper(localeName);
  result.isDefault = isDefaultLocale;
  result.locale = locale;
  return result;
}

String generateDelegateScript({
  List<Locale> supportedLocales,
  String objectClassName}) {

  final script = StringBuffer();

  void writeLine(String data) => script.writeln(data);

  writeLine('class AppLocalizationDelegate extends LocalizationsDelegate<$objectClassName> {');
  writeLine('  const AppLocalizationDelegate();');
  writeLine('');
  writeLine('  List<Locale> get supportedLocales {');
  writeLine('    return const <Locale>[');
  supportedLocales.forEach((locale) {
    if (locale.countryCode == null) {
      writeLine("      Locale('${locale.languageCode}'),");
    } else {
      writeLine("      Locale('${locale.languageCode}', '${locale.countryCode}'),");
    }
  });
  writeLine('    ];');
  writeLine('  }');
  writeLine('');
  writeLine(r'  LocaleListResolutionCallback listResolution({Locale fallback}) {');

  script
    ..writeln(r'    return (List<Locale> locales, Iterable<Locale> supported) {')
    ..writeln(r'      if (locales == null || locales.isEmpty) {')
    ..writeln(r'        return fallback ?? supported.first;')
    ..writeln(r'      } else {')
    ..writeln(r'        return _resolve(locales.first, fallback, supported);')
    ..writeln(r'      }')
    ..writeln(r'    };')
    ..writeln(r'  }')
    ..writeln()
    ..writeln(r'  LocaleResolutionCallback resolution({Locale fallback}) {')
    ..writeln(r'    return (Locale locale, Iterable<Locale> supported) {')
    ..writeln(r'      return _resolve(locale, fallback, supported);')
    ..writeln(r'    };')
    ..writeln(r'  }')
    ..writeln()
    ..writeln(r'  @override')
    ..writeln('  Future<$objectClassName> load(Locale locale) {')
    ..writeln(r'    final String lang = locale.toString();')
    ..writeln(r'    if (lang != null) {')
    ..writeln(r'      switch (lang) {');

  for (int i = 0; i < supportedLocales.length; i++) {
    final language = supportedLocales[i].toString();
    script
      ..writeln('        case "$language":')
      ..writeln('          $objectClassName.current = const \$$language();')
      ..writeln('          return SynchronousFuture<$objectClassName>($objectClassName.current);');
  }
  script
    ..writeln(r'        default:')
    ..writeln(r'          //...')
    ..writeln(r'      }')
    ..writeln(r'    }')
    ..writeln('    return SynchronousFuture<$objectClassName>(const $objectClassName());')
    ..writeln(r'  }')
    ..writeln();

  writeLine('  @override');
  writeLine('  bool isSupported(Locale locale) => _isSupported(locale);');
  writeLine('');
  writeLine('  @override');
  writeLine('  bool shouldReload(AppLocalizationDelegate old) => false;');
  writeLine('');

  script
  ..writeln(r'  /// Internal method to resolve a locale from a list of locales.')
  ..writeln(r'  Locale _resolve(Locale locale, Locale fallback, Iterable<Locale> supported) {')
  ..writeln(r'    if (locale == null || !_isSupported(locale)) {')
  ..writeln(r'      return fallback ?? supported.first;')
  ..writeln(r'    }')
  ..writeln()
  ..writeln(r'    final Locale languageLocale = Locale(locale.languageCode, null);')
  ..writeln(r'    if (supported.contains(locale)) {')
  ..writeln(r'      return locale;')
  ..writeln(r'    } else if (supported.contains(languageLocale)) {')
  ..writeln(r'      return languageLocale;')
  ..writeln(r'    } else {')
  ..writeln(r'      final Locale fallbackLocale = fallback ?? supported.first;')
  ..writeln(r'      return fallbackLocale;')
  ..writeln(r'    }')
  ..writeln(r'  }');
  writeLine('');
  writeLine('  bool _isSupported(Locale locale) {');
  writeLine('    if (locale != null) {');
  writeLine('      for (Locale supportedLocale in supportedLocales) {');
  writeLine('        if (supportedLocale.languageCode == locale.languageCode &&');
  writeLine('          supportedLocale.countryCode == locale.countryCode) {');
  writeLine('          return true;');
  writeLine('        }');
  writeLine('        if(supportedLocale.languageCode == locale.languageCode) {');
  writeLine('          return true;');
  writeLine('        }');
  writeLine('      }');
  writeLine('    }');
  writeLine('    return false;');
  writeLine('   }');
  writeLine('}');

  return script.toString();
}

String _firstCharUpper(String s) {
  return s.replaceRange(0, 1, s[0].toUpperCase());
}

const List<String> _rtl = <String>['ar', 'dv', 'fa', 'ha', 'he', 'iw', 'ji', 'ps', 'ur', 'yi'];
